/**
 * functions concerning motor selection for IndraSize
 *
 * @category client UI
 * @package core
 * @copyright  Copyright (c) 2011 Plan Software GmbH (http://www.plansoft.de)
 */

/**
 * start a direct motor selection or a full motor search
 *
 * @param strSearchButtonID - id of search button that issues this request
 * @param strMotorComboID - id of motor type combobox
 * @param strResultTableID - id of result table to store the search results into
 * @param bFull - full search(true) / direct selection (false)
 *
 * @return -
 */
startSearch = function( strSearchButtonID, strMotorComboID, strResultTableID, bFull )
{
	// if we have selected to search for a specific motor type, we don't need the whole ajax stuff
	if ( _testMotorType( strMotorComboID ) )
	{
		ekSubmit( strSearchButtonID );
		return;
	}
	
	// otherwise, start the whole searching process by the prepare-search request
	var strAjaxURL = '/ajax/';
	strAjaxURL += ( bFull ? 'fullSearch' : 'directSearch' );
	strAjaxURL += '/prepareSearch';
	
	if ( parent && parent.dialog )
		parent.dialog.disable();
	else dialog.disable();
	
	var strTabParam = null;
	if ( !bFull )
	{
		// for direct selection, we must also post the current selected tab
		var element = dialog[ 'IDC_SEARCH_LIST_TAB' ];
		if ( element )
		{
			strTabParam = '&IDC_SEARCH_LIST_TAB=' + element.selected;
			
			// for direct selection, we must also signal that the current working points has been used for the search
			var element = document.getElementById( 'WORKING_POINTS_CHANGED_' + element.selected );
			if ( element )
				element.value = 'false';
		}
	}
	ajaxHelper.startRequest( strAjaxURL, null, true, strTabParam, _prepareSearchHandler, [ strResultTableID, bFull ], _failureHandler );
}


/**
 * stop the search by telling the server to cleanup and stop the search
 * displays some error message, if needed
 *
 * @param strError - error message to display (may be null)
 * @param aParams - additional params (may be null)
 *                  aParams[0] - id of result table to store the search results into
 *                  aParams[1] - full search(true) / direct selection (false)
 *
 * @returns	-
 */
_stopSearch = function( strError, aParams )
{
	if ( parent && parent.dialog )
		parent.dialog.enable();
	else dialog.enable();
	
	// tell the server to stop the search
	var strAjaxURL = '/ajax/';
	strAjaxURL += ( aParams && aParams[1] ? 'fullSearch' : 'directSearch' );
	strAjaxURL += '/stopSearch';
	if ( aParams.length > 2 )
		aParams[2] = strError;
	else aParams.push( strError );
	ajaxHelper.startRequest( strAjaxURL, null, false, null, _stopSearchHandler, aParams, null );
	
	// ... and display the error message
	if ( strError )
		alert( strError );
}


/**
 * success handler for prepareSearch - request
 *
 * @param data - object with motor type list data for controlling the search from client side
 *                	error: error occured while trying to prepare the search
 * @param aParams - additional params
 *                  aParams[0] - id of result table to store the search results into
 *                  aParams[1] - full search(true) / direct selection (false)
 *
 * @returns	-
 */
_prepareSearchHandler = function( data, aParams )
{
	if ( !data )
	{
		_stopSearch( '', aParams );
		return;
	}
	
	// if an error occured on preparing the search, we display that error and stop the search immidiately
	// else we start the search be trying to get a motors list to search for
	if ( data.stopSearch )
	{
		_stopSearch( data.error, aParams );
		return;
	}
	tableHelper.clearTable( aParams[0], aParams[1] );
	
	var strAjaxURL = '/ajax/';
	strAjaxURL += ( aParams[1] ? 'fullSearch' : 'directSearch' );
	strAjaxURL += '/getMotors';
	ajaxHelper.startRequest( strAjaxURL, null, false, null, _startSearchHandler, aParams, _failureHandler );
}


/**
 * success handler for startSearch - request
 *
 * @param data - object with motor type list data for controlling the search from client side
 * 					motors: array with full motor types
 *                	error: error occured while trying to start the search
 * @param aParams - additional params
 *                  aParams[0] - id of result table to store the search results into
 *                  aParams[1] - full search(true) / direct selection (false)
 *
 * @returns	-
 */
_startSearchHandler = function( data, aParams )
{
	if ( !data )
	{
		_stopSearch( '', aParams );
		return;
	}
	
	// if an error occured, the motor type list will be empty
	// if an error occured on trying to get a motors list to search for, we display that error and stop the search immidiately
	if ( data.stopSearch || !data.motors )
	{
		// since preselection errors are reported with the failure report, we'd better not display the error so it won't appear twice
		_stopSearch( data.error, aParams );
		return;
	}
	
	// start the main search loop
	aParams.push( data.motors );	// motors to process
	aParams.push( 0 );				// index of motor in motor list that is processed

	var strAjaxURL = '/ajax/';
	strAjaxURL += ( aParams[1] ? 'fullSearch' : 'directSearch' );
	strAjaxURL += '/search';
	var strSearchParam = _composeSearchMotorParam( data.motors[0] );
	ajaxHelper.startRequest( strAjaxURL, null, false, strSearchParam, _searchHandler, aParams, _failureHandler );
}


/**
 * success handler for search - request
 *
 * @param data - object with motor type list data for controlling the search from client side
 *                  motorDataHeader: header (i.e. column names) for motor data (may be empty or missing)
 * 					motorData: array with full motor data for motor-regulator combinations from search results
 *                	error: error occured while trying to start the search (may be empty)
 * @param aParams - additional params
 *                  aParams[0] - id of result table to store the search results into
 *                  aParams[1] - full search(true) / direct selection (false)
 *                  aParams[2] - array with all motors to search
 *                  aParams[3] - index of motor in motor list that is processed
 *
 * @returns	-
 */
_searchHandler = function( data, aParams )
{
	if ( !data )
	{
		_stopSearch( '', aParams );
		return;
	}
	
	// the motor data may be empty - in most cases the motor failed some technical tests
	
	// if the header exists, this must be the very first search result, so we build the table header before inserting any rows
	if ( ( aParams[3] === 0 ) && data.motorDataHeader && ( data.motorDataHeader.length > 0 ) )
	{
		try {
			tableHelper.buildTableHeader( aParams[0], data.motorDataHeader );
		}
		catch( e )
		{
			alert( e.message );
		}
	}
	
	// display all resulting drives
	try {
		if ( data.motorData )
			tableHelper.appendToTable( aParams[0], data.motorData );
	}
	catch( e )
	{
		alert( e.message );
	}
	
	// if an error occured, the motor data will be empty
	// if an error occured on trying to get some motor data, we display that error and stop the search immidiately
	// we may also encounter the use case that the search in sum has resulted in more than the desired number of drives
	// here we must display the message and stop the search
	if ( data.stopSearch )
	{
		_stopSearch( data.error, aParams );
		return;
	}
	
	// the user may have cancelled the search on the progress bar, so we must stop the search loop
	if ( data.userCancel )
	{
		_stopSearch( '', aParams );
		return;
	}

	// continue the main search loop with the next motor in the list or stop the search if there are no more motors to process
	var nNextMotorIndex = ++aParams[3];
	if ( nNextMotorIndex < aParams[2].length )
	{
		var strAjaxURL = '/ajax/';
		strAjaxURL += ( aParams[1] ? 'fullSearch' : 'directSearch' );
		strAjaxURL += '/search';
		var aMotors = aParams[2];
		var strSearchParam = _composeSearchMotorParam( aMotors[nNextMotorIndex] );
		aParams[3] = nNextMotorIndex;
		ajaxHelper.startRequest( strAjaxURL, null, false, strSearchParam, _searchHandler, aParams, _failureHandler );
	}
	else _stopSearch( '', aParams );
}	


/**
 * success handler for prepareSearch - request
 *
 * @param data - object with motor type list data for controlling the search from client side
 *                	error: error occured while trying to prepare the search
 *
 * @param aParams - additional params
 *                  aParams[0] - id of result table to store the search results into
 *                  aParams[1] - full search(true) / direct selection (false)
 *                  aParams[2] - additional error message (may be empty)
 * 
 * @returns	-
 */
_stopSearchHandler = function( data, aParams )
{
	// the search has stopped, the result table is filled (or cleared)
	if ( !data )
		return;

	// drive selection is required by the server
	if ( ( aParams[0] !== '' ) && ( 'selectedDrive' in data ) && ( (data.selectedDrive-0) >= 0 ) )
		tableHelper.selectRow( aParams[0], (data.selectedDrive-0) );
	
	// we could popup the accumulated server error messages here
	if ( !data.errorFile || ( aParams[2] !== '' ) )
		return;
		
	// show textfile
	var newWindowAttributes = 'menubar=no toolbar=no,width=800,height=300';
    window.open( data.errorFile, '', newWindowAttributes );

	// uncomment for rtf/html
	// self.open( data.errorFile );
	document.getElementById( 'FAILURE_REPORT' ).value = '';
}


/**
 * failure handler for all search requests
 * ATTENTION: don't use this handler for stop requests
 *
 * @param data - object with some failure data
 *
 * @returns	-
 */
_failureHandler = function( data )
{
	_stopSearch();
}


/**
 * test if a specific motor type is selected
 *
 * @param strMotorComboID - id of motor type combobox
 *
 * @return boolean: true - specific motor type selected
 *                  false - 'all motors' selected
 */
_testMotorType = function( strMotorComboID )
{
	var elCombo = $( strMotorComboID );
	// the first element in motor family, motor size, motor type or regulator type combo box is always '_ALL_', if at least one element exists
	if ( elCombo && elCombo.options && ( elCombo.options.length > 0 ) )
		return( !elCombo.options[0].selected );
	
	return( false );
}


/**
 * compose post parameter to transfer to the server for a motor search
 *
 * @param strMotor - motor to search for
 *
 * @return string: requested parameter
 */
_composeSearchMotorParam = function( strMotor )
{
	var strParam = '&SEARCH_MOTOR=' + strMotor;
	// no others needed, the param is fetched directly from the post vars
	
	return( strParam );
}


