/**
 * functions commonly used in controllers
 *
 * @category   Controller
 * @package    Core
 * @copyright  Copyright (c) 2011 Plan Software GmbH (http://www.plansoft.de)
 */

// building the namespace
if ( typeof( Controller ) === 'undefined' )
	Controller = {};	

/**
 * tell the server to prepare a direct motor selection or a full motor search
 *
 * @param bFull - full search(true) / direct selection (false)
 *
 * @return object: error: error occured while trying to start the search (may be empty)
 */
Controller.prepareSearch = function( bFull )
{
	// prepare search (transfer control flow to C++ product logic)
	var pathData = 'TEMP.PREPARE_SEARCH';
	var strError = preparesearch( pathData, bFull );
	
	var result = {
		error: strError,
		stopSearch: Controller._isStopSearch( pathData )
	};
	
	return( result.toJSON() );
}


/**
 * build a list of motor types to search for
 *
 * @param bFull - full search(true) / direct selection (false)
 *
 * @return object: motors: array with full motor types
 *                 error: error occured while trying to start the search
 */
Controller.getMotors = function( bFull )
{
	// build motor list (transfer control flow to C++ product logic)
	var pathList = 'TEMP.MOTORLIST';
	var strError = getmotors( pathList, bFull );
	
	// build result object
	var aMotorlist = ( ( strError === '' ) ? Controller._getMotorListFromStore( pathList ) : [] );
		
	var result = {
		motors: aMotorlist,
		error: strError,
		stopSearch: Controller._isStopSearch( pathList )
	};
	delete( '', pathList );
	
	return( result.toJSON() );
}


/**
 * search for a motor
 *
 * @param bFull - full search(true) / direct selection (false)
 *
 * @return object: motors: array with full motor types
 *                 error: error occured while trying to start the search
 */
Controller.searchMotors = function( bFull )
{
	// get motor to search for
	var strMotor = getdlgvar( 'SEARCH_MOTOR' );

	// search for a motor (transfer control flow to C++ product logic)
	var pathResults = 'TEMP.SEARCH_RESULTS';
	delete( '', pathResults );
	var strError = searchmotors( pathResults, strMotor );
	
	// get searchresults from store
	var aHeader = Controller._getResultHeaderFromStore( pathResults );
	var aData = Controller._getResultDataFromStore( pathResults );
	var strUserCancelled = getstring( 'USER_CANCELLED', pathResults );
	
	var result = {
		motorDataHeader: aHeader,
		motorData: aData,
		userCancel: ( strUserCancelled === 'true' ),
		error: strError,
		stopSearch: Controller._isStopSearch( pathResults )
	};
	
	return( result.toJSON() );
}


/**
 * tell the server to stop a direct motor selection or a full motor search
 *
 * @param bFull - full search(true) / direct selection (false)
 *
 * @return object: error: error occured while trying to stop the search (may be empty)
 */
Controller.stopSearch = function( bFull )
{
	// stop search (transfer control flow to C++ product logic)
	var pathErrorFile = 'TEMP.STOP_SEARCH';
	var strError = stopsearch( pathErrorFile, bFull );
	var strErrorFile = getstring( 'ERROR_FILE', pathErrorFile );
	var nSelected = getint( 'SELECTED_DRIVE', pathErrorFile );
	delete( '', pathErrorFile );
	
	var result = {
		error: strError,
		errorFile: strErrorFile,
		selectedDrive: ( nSelected >= 0 ? nSelected : null )
	};
	
	return( result.toJSON() );
}


/**
 * get the motor types from a 'startSearch' action from the store as a string array
 *
 * @param pathList - store path for motor list
 *
 * @return array - the motor types (may be empty)
 */
Controller._getMotorListFromStore = function( pathList )
{
	var aMotors = [];

	var nCount = getsize( 'MOTOR', pathList ), strMotor;
	for( var i = 0; i < nCount; ++i )
	{
		strMotor = getstring( 'MOTOR[' + i + ']', pathList );
		aMotors.push( strMotor );
	}
	
	return( aMotors );
}


/**
 * get the column names from the search results.
 * Column names will only be produced as an additional search result on the first motor to process for the current search cycles
 *
 * @param pathResults - store path for search results
 *
 * @return array - the column names (may be empty)
 *                 null is no column names have been produced during the search 
 */
Controller._getResultHeaderFromStore = function( pathResults )
{
	var nCount = getsize( 'HEADER', pathResults );
	if ( nCount <= 0 )
		return( null );
		
	var aColumns = [], strColumn;
	for( var i = 0; i < nCount; ++i )
	{
		strColumn = getstring( 'HEADER[' + i + ']', pathResults );
		aColumns.push( strColumn );
	}
	
	return( aColumns );
}


/**
 * get the search results.
 * Column names will only be produced as an additional search result on the first motor to process for the current search cycles
 *
 * @param pathResults - store path for search results
 *
 * @return array - the search results (may be empty) - each array index contains an array with all data for a single drive
 *                 null is no search results have been produced during the search 
 */
Controller._getResultDataFromStore = function( pathResults )
{
	var nDriveCount = getsize( 'RESULTS', pathResults );
	if ( nDriveCount <= 0 )
		return( null );
		
	var aResults = [], aAttributes, nAttributeCount, pathDrive, strAttribute;
	for( var i = 0; i < nDriveCount; ++i )
	{
		pathDrive = pathResults + '.RESULTS[' + i + ']'
		nAttributeCount = getint( 'COUNT', pathDrive );
		if ( nAttributeCount <= 0 )
			continue;
		
		aAttributes = [];
		for( var j = 0; j < nAttributeCount; ++j )
		{
			strAttribute = getstring( 'ATTRIBUTE[' + j + ']', pathDrive );
			aAttributes.push( strAttribute );
		}
		
		aResults.push( aAttributes );
	}
	
	return( aResults );
}


/**
 * test if the search should be stopped
 *
 * @param pathResults - store path for flag
 *
 * @return boolean - true: stop the search process
 *                   false: continue the search process
 */
Controller._isStopSearch = function( pathResults )
{
	var strValue = getstring( 'STOP_SEARCH', pathResults );
	return( strValue == 'true' );
}

